==> driverlessai-1.8.7.2.post1-py3-none-any.whl: driverlessai/_core.py <==
"""Official Python client for Driverless AI."""

import copy
import importlib
import json
import os
import re
from typing import Any, Union

import requests

from driverlessai import __version__
from driverlessai import _datasets
from driverlessai import _experiments
from driverlessai import _recipes
from driverlessai import _server
from driverlessai import _utils


###############################
# Custom Exceptions
###############################


# If we're not able to communicate with the DAI
# server, this exception is thrown.
class ServerDownException(_utils.ClientException):
    pass


# If we're not able to scrape server version
class ServerVersionExtractionFailed(_utils.ClientException):
    pass


class ServerVersionNotSupported(_utils.ClientException):
    pass


###############################
# Helper Functions
###############################


def is_server_up(
    address: str, timeout: int = 10, verify: Union[bool, str] = False
) -> bool:
    """Checks if a Driverless AI server is running.

    Args:
        address: full URL of the Driverless AI server to connect to
        timeout: timeout if the server has not issued a response in this many seconds
        verify: when using https on the Driverless AI server, setting this to
            False will disable SSL certificates verification. A path to
            cert(s) can also be passed to verify, see:
            https://requests.readthedocs.io/en/master/user/advanced/#ssl-cert-verification
    """
    try:
        return requests.get(address, timeout=timeout, verify=verify).status_code == 200
    except (requests.exceptions.ConnectionError, requests.exceptions.Timeout):
        return False


###############################
# DAI Python Client
###############################


class Client:
    """Connect to and interact with a Driverless AI server.

    Args:
        address: full URL of the Driverless AI server to connect to
        username: username for authentication on the Driverless AI server
        password: password for authentication on the Driverless AI server
        verify: when using https on the Driverless AI server, setting this to
            False will disable SSL certificates verification. A path to
            cert(s) can also be passed to verify, see:
            https://requests.readthedocs.io/en/master/user/advanced/#ssl-cert-verification
        backend_version_override: version of client backend to use, overrides
            Driverless AI server version detection. Specify ``"latest"`` to get
            the most recent backend supported. In most cases the user should
            rely on Driverless AI server version detection and leave this as
            the default ``None``.

    Attributes:
        connectors (Connectors): for interacting with connectors on the
            Driverless AI server
        datasets (Datasets): for interacting with datasets on the
            Driverless AI server
        experiments (Experiments): for interacting with experiments on the
            Driverless AI server
        recipes (Recipes): for interacting with recipes on the
            Driverless AI server
        server (Server): for getting information about the Driverless AI server
    """

    def __init__(
        self,
        address: str,
        username: str,
        password: str,
        verify: Union[bool, str] = True,
        backend_version_override: str = None,
    ) -> None:
        # We should check if the server is up first.
        # If we're unable to ping it, we fail.
        if not is_server_up(address, verify=verify):
            if address.startswith("https"):
                raise ServerDownException(
                    "Unable to communicate with Driverless AI server. "
                    "Please make sure the server is running, "
                    "the address is correct, and `verify` is specified."
                )
            raise ServerDownException(
                "Unable to communicate with Driverless AI server. "
                "Please make sure the server is running and the address is correct."
            )

        if backend_version_override is None:
            # If the response status was not 200, we are dealing with a version of DAI
            # older than 1.8.2. If we got a response other than 200 we attempt
            # to extract the version by scraping the login page.
            try:
                response = requests.get(address + "/serverversion", verify=verify)
                if response.status_code == 200:
                    json_response = json.loads(response.text)
                    server_version = json_response["result"]["version"]
                else:
                    server_version = re.search(
                        "DRIVERLESS AI ([0-9.]+)",
                        requests.get(address, verify=verify).text,
                    )[1]
            except Exception:
                raise ServerVersionExtractionFailed(
                    "Unable to extract server version. "
                    "Please make sure the address is correct."
                )
        else:
            if backend_version_override == "latest":
                backend_version_override = re.search("[0-9.]+", __version__)[0]
            server_version = backend_version_override

        server_module_path = (
            f"driverlessai._h2oai_client_{server_version.replace('.', '_')}"
        )
        try:
            self._server_module = importlib.import_module(
                server_module_path
            )  # type: Any
        except ModuleNotFoundError:
            raise ServerVersionNotSupported(
                f"Server version {server_version} is not supported, "
                "try updating to the latest client."
            )

        self._backend = self._server_module.protocol.Client(
            address=address, username=username, password=password, verify=verify
        )

        self.connectors = _datasets.Connectors(self)
        self.datasets = _datasets.Datasets(self)
        self.experiments = _experiments.Experiments(self)
        self.recipes = _recipes.Recipes(self)
        self.server = _server.Server(
            self, address, username, self._backend.get_app_version().version
        )

        # Warns if license is missing/expired/about to expire
        self.server.license.is_valid()

    def __repr__(self) -> str:
        return f"{self.__class__} {self!s}"

    def __str__(self) -> str:
        return self.server.address

    def _download(
        self,
        server_path: str,
        dst_dir: str,
        overwrite: bool = False,
        timeout: float = 5,
        verbose: bool = True,
    ) -> str:
        dst_path = os.path.join(dst_dir, os.path.basename(server_path))
        if overwrite or not os.path.exists(dst_path):
            s = requests.Session()
            url = self.server.address + "/files/" + server_path
            if hasattr(self._backend, "_session"):
                cookies = copy.deepcopy(self._backend._session.cookies)
                verify = copy.deepcopy(self._backend._session.verify)
            else:
                cookies = copy.deepcopy(self._backend._cookies)
                verify = copy.deepcopy(self._backend._verify)
            res = s.get(url, cookies=cookies, verify=verify, timeout=timeout)
            res.raise_for_status()
            with open(dst_path, "wb") as f:
                f.write(res.content)
        else:
            raise FileExistsError(
                dst_path + " already exists. Use `overwrite` to force download."
            )
        if verbose:
            print("Downloaded '", dst_path, "'", sep="")
        return dst_path

==> driverlessai-1.9.0.1-py3-none-any.whl: driverlessai/_core.py <==
"""Official Python client for Driverless AI."""

import copy
import importlib
import json
import os
import re
from typing import Any
from typing import Union

import requests

from driverlessai import __version__
from driverlessai import _datasets
from driverlessai import _experiments
from driverlessai import _recipes
from driverlessai import _server
from driverlessai import _utils


###############################
# Custom Exceptions
###############################


# If we're not able to communicate with the DAI
# server, this exception is thrown.
class ServerDownException(_utils.ClientException):
    pass


# If we're not able to scrape server version
class ServerVersionExtractionFailed(_utils.ClientException):
    pass


class ServerVersionNotSupported(_utils.ClientException):
    pass


###############################
# Helper Functions
###############################


def is_server_up(
    address: str, timeout: int = 10, verify: Union[bool, str] = False
) -> bool:
    """Checks if a Driverless AI server is running.

    Args:
        address: full URL of the Driverless AI server to connect to
        timeout: timeout if the server has not issued a response in this many seconds
        verify: when using https on the Driverless AI server, setting this to
            False will disable SSL certificates verification. A path to
            cert(s) can also be passed to verify, see:
            https://requests.readthedocs.io/en/master/user/advanced/#ssl-cert-verification

    Examples::

        driverlessai.is_server_up(
            address='http://localhost:12345',
        )
    """
    try:
        return requests.get(address, timeout=timeout, verify=verify).status_code == 200
    except (requests.exceptions.ConnectionError, requests.exceptions.Timeout):
        return False


###############################
# DAI Python Client
###############################


class Client:
    """Connect to and interact with a Driverless AI server.

    Args:
        address: full URL of the Driverless AI server to connect to
        username: username for authentication on the Driverless AI server
        password: password for authentication on the Driverless AI server
        verify: when using https on the Driverless AI server, setting this to
            False will disable SSL certificates verification. A path to
            cert(s) can also be passed to verify, see:
            https://requests.readthedocs.io/en/master/user/advanced/#ssl-cert-verification
        backend_version_override: version of client backend to use, overrides
            Driverless AI server version detection. Specify ``"latest"`` to get
            the most recent backend supported. In most cases the user should
            rely on Driverless AI server version detection and leave this as
            the default ``None``.

    Attributes:
        connectors (Connectors): for interacting with connectors on the
            Driverless AI server
        datasets (Datasets): for interacting with datasets on the
            Driverless AI server
        experiments (Experiments): for interacting with experiments on the
            Driverless AI server
        recipes (Recipes): for interacting with recipes on the
            Driverless AI server
        server (Server): for getting information about the Driverless AI server

    Examples::

        dai = driverlessai.Client(
            address='http://localhost:12345',
            username='py',
            password='py'
        )
    """

    def __init__(
        self,
        address: str,
        username: str,
        password: str,
        verify: Union[bool, str] = True,
        backend_version_override: str = None,
    ) -> None:
        address = address.rstrip("/")

        # Check if the server is up, if we're unable to ping it we fail.
        if not is_server_up(address, verify=verify):
            if address.startswith("https"):
                raise ServerDownException(
                    "Unable to communicate with Driverless AI server. "
                    "Please make sure the server is running, "
                    "the address is correct, and `verify` is specified."
                )
            raise ServerDownException(
                "Unable to communicate with Driverless AI server. "
                "Please make sure the server is running and the address is correct."
            )

        # Try to get server version, if we can't we fail.
        if backend_version_override is None:
            server_version = self._detect_server_version(address, verify)
        else:
            if backend_version_override == "latest":
                backend_version_override = re.search("[0-9.]+", __version__)[0].rstrip(
                    "."
                )
            server_version = backend_version_override

        # Import backend that matches server version, if we can't we fail.
        server_module_path = (
            f"driverlessai._h2oai_client_{server_version.replace('.', '_')}"
        )
        try:
            self._server_module: Any = importlib.import_module(server_module_path)
        except ModuleNotFoundError:
            raise ServerVersionNotSupported(
                f"Server version {server_version} is not supported, "
                "try updating to the latest client."
            )

        self._backend = self._server_module.protocol.Client(
            address=address, username=username, password=password, verify=verify
        )
        if server_version[:3] == "1.9":
            self._gui_sep = "/#/"
        if server_version[:3] == "1.8":
            self._gui_sep = "/#"
        self.connectors = _datasets.Connectors(self)
        self.datasets = _datasets.Datasets(self)
        self.experiments = _experiments.Experiments(self)
        self.recipes = _recipes.Recipes(self)
        self.server = _server.Server(
            self, address, username, self._backend.get_app_version().version
        )

        # Warns if license is missing/expired/about to expire
        self.server.license.is_valid()

    def __repr__(self) -> str:
        return f"{self.__class__} {self!s}"

    def __str__(self) -> str:
        return self.server.address

    @staticmethod
    def _detect_server_version(address: str, verify: Union[bool, str]) -> str:
        """Trys multiple methods to retrieve server version."""
        # query server version endpoint
        response = requests.get(f"{address}/serverversion", verify=verify)
        if response.status_code == 200:
            return re.search("[0-9.]+", response.text)[0]
        # extract the version by scraping the login page
        response = requests.get(address, verify=verify)
        scrapings = re.search("DRIVERLESS AI ([0-9.]+)", response.text)
        if scrapings:
            return scrapings[1]
        # if login is disabled, get cookie and make rpc call
        with requests.Session() as s:
            s.get(f"{address}/login", verify=verify)
            response = s.post(
                f"{address}/rpc",
                data=json.dumps(
                    {"id": "", "method": "api_get_app_version", "params": {}}
                ),
            )
            server_version = response.json()["result"]["version"]
        if server_version:
            return server_version
        # fail
        raise ServerVersionExtractionFailed(
            "Unable to extract server version. "
            "Please make sure the address is correct."
        )

    def _download(
        self,
        server_path: str,
        dst_dir: str,
        overwrite: bool = False,
        timeout: float = 5,
        verbose: bool = True,
    ) -> str:
        dst_path = os.path.join(dst_dir, os.path.basename(server_path))
        if overwrite or not os.path.exists(dst_path):
            s = requests.Session()
            url = self.server.address + "/files/" + server_path
            if hasattr(self._backend, "_session"):
                cookies = copy.deepcopy(self._backend._session.cookies)
                verify = copy.deepcopy(self._backend._session.verify)
            else:
                cookies = copy.deepcopy(self._backend._cookies)
                verify = copy.deepcopy(self._backend._verify)
            res = s.get(url, cookies=cookies, verify=verify, timeout=timeout)
            res.raise_for_status()
            with open(dst_path, "wb") as f:
                f.write(res.content)
        else:
            raise FileExistsError(
                dst_path + " already exists. Use `overwrite` to force download."
            )
        if verbose:
            print("Downloaded '", dst_path, "'", sep="")
        return dst_path
